@@ -90,6 +90,10 @@ enum {
 	CHART_PROP_CARDINALITY_VALID,
 	CHART_PROP_PLOT_AREA,
 	CHART_PROP_PLOT_AREA_IS_MANUAL,
+	CHART_PROP_X_POS,
+	CHART_PROP_Y_POS,
+	CHART_PROP_ROWS,
+	CHART_PROP_COLUMNS
 };
 
 static GType gog_chart_view_get_type (void);
@@ -128,6 +132,7 @@ gog_chart_set_property (GObject *obj, guint param_id,
 	GogChart *chart = GOG_CHART (obj);
 	char **str_doubles;
 	char const *str;
+	gboolean changed = FALSE;
 
 	switch (param_id) {
 	case CHART_PROP_PLOT_AREA:
@@ -146,9 +151,30 @@ gog_chart_set_property (GObject *obj, guint param_id,
 	case CHART_PROP_PLOT_AREA_IS_MANUAL:
 		chart->is_plot_area_manual = g_value_get_boolean (value);
 		break;
+	case CHART_PROP_X_POS:
+		chart->x_pos = g_value_get_int (value);
+		changed = TRUE;
+		break;
+	case CHART_PROP_Y_POS:
+		chart->y_pos = g_value_get_int (value);
+		changed = TRUE;
+		break;
+	case CHART_PROP_COLUMNS:
+		chart->cols = g_value_get_int (value);
+		changed = TRUE;
+		break;
+	case CHART_PROP_ROWS:
+		chart->rows = g_value_get_int (value);
+		changed = TRUE;
+		break;
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
 		 return; /* NOTE : RETURN */
 	}
+
+	if (changed) {
+		gog_graph_validate_chart_layout (GOG_GRAPH (GOG_OBJECT (chart)->parent));
+		gog_object_emit_changed (GOG_OBJECT (obj), TRUE);
+	}
 }
 
 static void
@@ -178,12 +204,97 @@ gog_chart_get_property (GObject *obj, guint param_id,
 	case CHART_PROP_PLOT_AREA_IS_MANUAL:
 		g_value_set_boolean (value, chart->is_plot_area_manual);
 		break;
+	case CHART_PROP_X_POS:
+		g_value_set_int (value, chart->x_pos);
+		break;
+	case CHART_PROP_Y_POS:
+		g_value_set_int (value, chart->y_pos);
+		break;
+	case CHART_PROP_COLUMNS:
+		g_value_set_int (value, chart->cols);
+		break;
+	case CHART_PROP_ROWS:
+		g_value_set_int (value, chart->rows);
+		break;
 
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
 		 break;
 	}
 }
 
+#ifdef GOFFICE_WITH_GTK
+typedef struct {
+	GtkBuilder	*gui;
+	GtkWidget	*x_spin, *y_spin, *w_spin, *h_spin;
+	gulong		 w_spin_signal, h_spin_signal;
+	GtkWidget	*position_select_combo;
+	GtkWidget	*manual_setting_table;
+	GogChart	*chart;
+} PlotAreaPrefState;
+
+static void
+plot_area_pref_state_free (PlotAreaPrefState *state)
+{
+	g_object_unref (state->chart);
+	g_object_unref (state->gui);
+}
+
+static void
+cb_plot_area_changed (GtkWidget *spin, PlotAreaPrefState *state)
+{
+	GogViewAllocation pos;
+	double value;
+	double max;
+
+       	value = gtk_spin_button_get_value (GTK_SPIN_BUTTON (spin)) / 100.0;
+
+       	gog_chart_get_plot_area (state->chart, &pos);
+	if (spin == state->x_spin) {
+		pos.x = value;
+		max = 1.0 - pos.x;
+		g_signal_handler_block (state->w_spin, state->w_spin_signal);
+		gtk_spin_button_set_range (GTK_SPIN_BUTTON (state->w_spin), 0.0, max * 100.0);
+		if (pos.w > max) pos.w = max;
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (state->w_spin), pos.w * 100.0);
+		g_signal_handler_unblock (state->w_spin, state->w_spin_signal);
+	}
+	else if (spin == state->y_spin) {
+		pos.y = value;
+		max = 1.0 - pos.y;
+		g_signal_handler_block (state->h_spin, state->h_spin_signal);
+		gtk_spin_button_set_range (GTK_SPIN_BUTTON (state->h_spin), 0.0, max * 100.0);
+		if (pos.h > max) pos.h = max;
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (state->h_spin), pos.w * 100.0);
+		g_signal_handler_unblock (state->h_spin, state->h_spin_signal);
+	}
+	else if (spin == state->w_spin) {
+		pos.w = value;
+	}
+	else if (spin == state->h_spin) {
+		pos.h = value;
+	}
+	gog_chart_set_plot_area (state->chart, &pos);
+	gtk_combo_box_set_active (GTK_COMBO_BOX (state->position_select_combo), 1);
+	gtk_widget_show (state->manual_setting_table);
+}
+
+static void
+cb_manual_position_changed (GtkComboBox *combo, PlotAreaPrefState *state)
+{
+	if (gtk_combo_box_get_active (combo) == 1) {
+		GogViewAllocation plot_area;
+
+		gog_chart_get_plot_area (state->chart, &plot_area);
+		gog_chart_set_plot_area (state->chart, &plot_area);
+		gtk_widget_show (state->manual_setting_table);
+	} else {
+		gog_chart_set_plot_area (state->chart, NULL);
+		gtk_widget_hide (state->manual_setting_table);
+	}
+}
+
+#endif
+
 static void
 gog_chart_populate_editor (GogObject *gobj,
 			   GOEditor *editor,
@@ -192,8 +303,69 @@ gog_chart_populate_editor (GogObject *gobj,
 {
 	static guint chart_pref_page = 0;
 
+	GtkBuilder *gui;
+	PlotAreaPrefState *state;
+	gboolean is_plot_area_manual;
+	GogViewAllocation plot_area;
+	GtkWidget *w;
+	GogChart *chart = GOG_CHART (gobj);
+
+	g_return_if_fail (chart != NULL);
+
+	gui = go_gtk_builder_new ("gog-plot-prefs.ui", GETTEXT_PACKAGE, cc);
+	g_return_if_fail (gui != NULL);
+
 	(GOG_OBJECT_CLASS(chart_parent_klass)->populate_editor) (gobj, editor, dalloc, cc);
 
+	state = g_new  (PlotAreaPrefState, 1);
+	state->chart = chart;
+	state->gui = gui;
+
+	g_object_ref (G_OBJECT (chart));
+	is_plot_area_manual = gog_chart_get_plot_area (chart, &plot_area);
+
+	state->x_spin = go_gtk_builder_get_widget (gui, "x_spin");
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON (state->x_spin),
+				   plot_area.x * 100.0);
+	g_signal_connect (G_OBJECT (state->x_spin), "value-changed",
+			  G_CALLBACK (cb_plot_area_changed), state);
+
+	state->y_spin = go_gtk_builder_get_widget (gui, "y_spin");
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON (state->y_spin),
+				   plot_area.y * 100.0);
+	g_signal_connect (G_OBJECT (state->y_spin), "value-changed",
+			  G_CALLBACK (cb_plot_area_changed), state);
+
+	state->w_spin = go_gtk_builder_get_widget (gui, "w_spin");
+	gtk_spin_button_set_range (GTK_SPIN_BUTTON (state->w_spin),
+				   0.0, (1.0 - plot_area.x) * 100.0);
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON (state->w_spin),
+				   100.0 * plot_area.w);
+	state->w_spin_signal = g_signal_connect (G_OBJECT (state->w_spin), "value-changed",
+						 G_CALLBACK (cb_plot_area_changed), state);
+
+	state->h_spin = go_gtk_builder_get_widget (gui, "h_spin");
+	gtk_spin_button_set_range (GTK_SPIN_BUTTON (state->h_spin),
+				   0.0, (1.0 - plot_area.y) * 100.0);
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON (state->h_spin),
+				   100.0 * plot_area.h);
+	state->h_spin_signal = g_signal_connect (G_OBJECT (state->h_spin), "value-changed",
+						 G_CALLBACK (cb_plot_area_changed), state);
+
+	state->position_select_combo = go_gtk_builder_get_widget (gui, "position_select_combo");
+	gtk_combo_box_set_active (GTK_COMBO_BOX (state->position_select_combo),
+				  is_plot_area_manual ? 1 : 0);
+	state->manual_setting_table = go_gtk_builder_get_widget (gui, "manual_setting_table");
+	if (!is_plot_area_manual)
+		gtk_widget_hide (state->manual_setting_table);
+
+	g_signal_connect (G_OBJECT (state->position_select_combo),
+			  "changed", G_CALLBACK (cb_manual_position_changed), state);
+
+	w = go_gtk_builder_get_widget (gui, "gog_plot_prefs");
+	g_signal_connect_swapped (G_OBJECT (w), "destroy", G_CALLBACK (plot_area_pref_state_free), state);
+	go_editor_add_page (editor, w, _("Plot area"));
+
 	go_editor_set_store_page (editor, &chart_pref_page);
 }
 
@@ -401,42 +573,42 @@ static gboolean role_3d_box_can_add	(GogObject const *parent) {return FALSE;}
 static gboolean role_3d_box_can_remove	(GogObject const *parent) {return FALSE;}
 
 static GogObjectRole const roles[] = {
-	{ N_("Backplane"), "GogGrid",	0,
+	{ N_("Backplane"), "GogGrid",	1,
 	  GOG_POSITION_SPECIAL, GOG_POSITION_SPECIAL, GOG_OBJECT_NAME_BY_ROLE,
 	  role_grid_can_add, NULL, NULL, role_grid_post_add, role_grid_pre_remove, NULL, { -1 } },
-	{ N_("XY-Backplane"), "GogGrid",	0,
+	{ N_("XY-Backplane"), "GogGrid",	1,
 	  GOG_POSITION_SPECIAL, GOG_POSITION_SPECIAL, GOG_OBJECT_NAME_BY_ROLE,
 	  xy_grid_3d_can_add, NULL, NULL, xy_grid_3d_post_add, NULL, NULL,
 	  { GOG_GRID_XY } },
-	{ N_("YZ-Backplane"), "GogGrid",	0,
+	{ N_("YZ-Backplane"), "GogGrid",	1,
 	  GOG_POSITION_SPECIAL, GOG_POSITION_SPECIAL, GOG_OBJECT_NAME_BY_ROLE,
 	  yz_grid_3d_can_add, NULL, NULL, yz_grid_3d_post_add, NULL, NULL,
 	  { GOG_GRID_YZ } },
-	{ N_("ZX-Backplane"), "GogGrid",	0,
+	{ N_("ZX-Backplane"), "GogGrid",	1,
 	  GOG_POSITION_SPECIAL, GOG_POSITION_SPECIAL, GOG_OBJECT_NAME_BY_ROLE,
 	  zx_grid_3d_can_add, NULL, NULL, zx_grid_3d_post_add, NULL, NULL,
 	  { GOG_GRID_ZX } },
-	{ N_("X-Axis"), "GogAxis",	1,
+	{ N_("X-Axis"), "GogAxis",	2,
 	  GOG_POSITION_PADDING, GOG_POSITION_PADDING, GOG_OBJECT_NAME_BY_ROLE,
 	  x_axis_can_add, axis_can_remove, NULL, x_axis_post_add, axis_pre_remove, NULL,
 	  { GOG_AXIS_X } },
-	{ N_("Y-Axis"), "GogAxis",	2,
+	{ N_("Y-Axis"), "GogAxis",	3,
 	  GOG_POSITION_PADDING, GOG_POSITION_PADDING, GOG_OBJECT_NAME_BY_ROLE,
 	  y_axis_can_add, axis_can_remove, NULL, y_axis_post_add, axis_pre_remove, NULL,
 	  { GOG_AXIS_Y } },
-	{ N_("Z-Axis"), "GogAxis",	3,
+	{ N_("Z-Axis"), "GogAxis",	4,
 	  GOG_POSITION_PADDING, GOG_POSITION_PADDING, GOG_OBJECT_NAME_BY_ROLE,
 	  z_axis_can_add, axis_can_remove, NULL, z_axis_post_add, axis_pre_remove, NULL,
 	  { GOG_AXIS_Z } },
-	{ N_("Circular-Axis"), "GogAxis", 1,
+	{ N_("Circular-Axis"), "GogAxis", 2,
 	  GOG_POSITION_PADDING, GOG_POSITION_PADDING, GOG_OBJECT_NAME_BY_ROLE,
 	  circular_axis_can_add, axis_can_remove, NULL, circular_axis_post_add, axis_pre_remove, NULL,
 	  { GOG_AXIS_CIRCULAR } },
-	{ N_("Radial-Axis"), "GogAxis",	2,
+	{ N_("Radial-Axis"), "GogAxis",	3,
 	  GOG_POSITION_PADDING, GOG_POSITION_PADDING, GOG_OBJECT_NAME_BY_ROLE,
 	  radial_axis_can_add, axis_can_remove, NULL, radial_axis_post_add, axis_pre_remove, NULL,
 	  { GOG_AXIS_RADIAL } },
-	{ N_("Pseudo-3D-Axis"), "GogAxis", 3,
+	{ N_("Pseudo-3D-Axis"), "GogAxis", 4,
 	  GOG_POSITION_SPECIAL, GOG_POSITION_SPECIAL, GOG_OBJECT_NAME_BY_ROLE,
 	  pseudo_3d_axis_can_add, axis_can_remove, NULL, pseudo_3d_axis_post_add, axis_pre_remove, NULL,
 	  { GOG_AXIS_PSEUDO_3D } },
@@ -444,14 +616,14 @@ static GogObjectRole const roles[] = {
 	  GOG_POSITION_SPECIAL, GOG_POSITION_SPECIAL, GOG_OBJECT_NAME_BY_ROLE,
 	  bubble_axis_can_add, axis_can_remove, NULL, bubble_axis_post_add, axis_pre_remove, NULL,
 	  { GOG_AXIS_BUBBLE } },
-	{ N_("Color-Axis"), "GogAxis", 5,
+	{ N_("Color-Axis"), "GogAxis", 4,
 	  GOG_POSITION_SPECIAL, GOG_POSITION_SPECIAL, GOG_OBJECT_NAME_BY_ROLE,
 	  color_axis_can_add, axis_can_remove, NULL, color_axis_post_add, axis_pre_remove, NULL,
 	  { GOG_AXIS_COLOR } },
 	{ N_("Plot"), "GogPlot",	6,	/* keep the axis before the plots */
 	  GOG_POSITION_SPECIAL, GOG_POSITION_SPECIAL, GOG_OBJECT_NAME_BY_TYPE,
 	  role_plot_can_add, NULL, NULL, role_plot_post_add, role_plot_pre_remove, NULL, { -1 } },
-	{ N_("Title"), "GogLabel",	10,
+	{ N_("Title"), "GogLabel",	0,
 	  GOG_POSITION_COMPASS|GOG_POSITION_ANY_MANUAL,
 	  GOG_POSITION_N|GOG_POSITION_ALIGN_CENTER,
 	  GOG_OBJECT_NAME_BY_ROLE,
@@ -468,7 +640,7 @@ static GogObjectRole const roles[] = {
 	  GOG_OBJECT_NAME_BY_ROLE,
 	  NULL, NULL, NULL, NULL, NULL, NULL, { -1 } },
 #endif
-	{ N_("3D-Box"), "Gog3DBox",	0,
+	{ N_("3D-Box"), "Gog3DBox",	1,
 	  GOG_POSITION_SPECIAL, GOG_POSITION_SPECIAL, GOG_OBJECT_NAME_BY_ROLE,
 	  role_3d_box_can_add, role_3d_box_can_remove, NULL, NULL, NULL, NULL, { -1 } }
 };
@@ -505,6 +677,23 @@ gog_chart_class_init (GogObjectClass *gog_klass)
 				      _("Is plot area manual"),
 				      FALSE,
 				      GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
+	g_object_class_install_property (gobject_klass, CHART_PROP_X_POS,
+		g_param_spec_int ("xpos", _("xpos"),
+			_("Horizontal chart position in graph grid"),
+			0, G_MAXINT, 0, G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
+	/* we need to force saving of ypos since the default is not constant */
+	g_object_class_install_property (gobject_klass, CHART_PROP_Y_POS,
+		g_param_spec_int ("ypos", _("ypos"),
+			_("Vertical chart position in graph grid"),
+			0, G_MAXINT, 0, G_PARAM_READWRITE | GO_PARAM_PERSISTENT | GOG_PARAM_FORCE_SAVE));
+	g_object_class_install_property (gobject_klass, CHART_PROP_COLUMNS,
+		g_param_spec_int ("columns", _("columns"),
+			_("Number of columns in graph grid"),
+			1, G_MAXINT, 1, G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
+	g_object_class_install_property (gobject_klass, CHART_PROP_ROWS,
+		g_param_spec_int ("rows", _("rows"),
+			_("Number of rows in graph grid"),
+			1, G_MAXINT, 1, G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 
 	gog_klass->view_type = gog_chart_view_get_type ();
 	gog_klass->update    = gog_chart_update;
@@ -515,10 +704,12 @@ gog_chart_class_init (GogObjectClass *gog_klass)
 static void
 gog_chart_init (GogChart *chart)
 {
-	chart->x     = 0;
-	chart->y     = 0;
-	chart->cols  = 0;
-	chart->rows  = 0;
+	chart->x_pos =
+	chart->y_pos = 
+	chart->cols  = 		
+	chart->rows  =
+	chart->x_pos_actual = 
+	chart->y_pos_actual = 0;
 
 	/* start as true so that we can queue an update when it changes */
 	chart->cardinality_valid = TRUE;
@@ -554,8 +745,8 @@ gog_chart_get_position (GogChart const *chart,
 	if (chart->cols <= 0 || chart->rows <= 0)
 		return FALSE;
 
-	if (x != NULL)	  *x	= chart->x;
-	if (y != NULL)	  *y	= chart->y;
+	if (x != NULL)	  *x	= chart->x_pos;
+	if (y != NULL)	  *y	= chart->y_pos;
 	if (cols != NULL) *cols	= chart->cols;
 	if (rows != NULL) *rows	= chart->rows;
 
@@ -577,12 +768,12 @@ gog_chart_set_position (GogChart *chart,
 {
 	g_return_if_fail (GOG_IS_CHART (chart));
 
-	if (chart->x == x && chart->y == y &&
+	if (chart->x_pos == x && chart->y_pos == y &&
 	    chart->cols == cols && chart->rows == rows)
 		return;
 
-	chart->x = x;
-	chart->y = y;
+	chart->x_pos = x;
+	chart->y_pos = y;
 	chart->cols = cols;
 	chart->rows = rows;
 
@@ -1155,7 +1346,7 @@ gog_chart_view_render (GogView *view, GogViewAllocation const *bbox)
 	if (gog_chart_is_3d (chart)) {
 		for (ptr = view->children ; ptr != NULL ; ptr = ptr->next) {
 			child_view = ptr->data;
-			if (!GOG_IS_AXIS (child_view->model) && !GOG_IS_PLOT (child_view->model))
+			if (!GOG_IS_AXIS (child_view->model) && !GOG_IS_PLOT (child_view->model) && !GOG_IS_LABEL (child_view->model))
 				gog_view_render	(ptr->data, bbox);
 		}
 		/* now render plot and axes */
@@ -1168,10 +1359,9 @@ gog_chart_view_render (GogView *view, GogViewAllocation const *bbox)
 		}
 		for (ptr = view->children ; ptr != NULL ; ptr = ptr->next) {
 			child_view = ptr->data;
-			if (GOG_IS_AXIS (child_view->model))
+			if (!GOG_IS_PLOT (child_view->model))
 				continue;
-			if (GOG_IS_PLOT (child_view->model))
-				gog_view_render	(ptr->data, bbox);
+			gog_view_render	(ptr->data, bbox);
 		}
 	} else {
 		/* KLUDGE: render grid lines before axis */
@@ -1185,10 +1375,16 @@ gog_chart_view_render (GogView *view, GogViewAllocation const *bbox)
 			if (GOG_IS_PLOT (child_view->model)) {
 			    if (!GOG_PLOT (child_view->model)->render_before_axes)
 				gog_view_render	(ptr->data, bbox);
-			} else
+			} else if (!GOG_IS_LABEL (child_view->model))
 				gog_view_render	(ptr->data, bbox);
 		}
 	}
+	for (ptr = view->children ; ptr != NULL ; ptr = ptr->next) {
+		child_view = ptr->data;
+		if (!GOG_IS_LABEL (child_view->model))
+			continue;
+		gog_view_render	(ptr->data, bbox);
+	}
 }
 
 static void
