@@ -36,9 +36,13 @@ enum {
 	LAST_SIGNAL
 };
 
-/* GogStyle object: this is a mere copy of GOStyle but is needed for file
- format compatibility */
-typedef GOStyle GogStyle;
+/**GogStyle:
+ * This is a mere copy of GOStyle but is needed for file
+ * format compatibility
+ **/
+struct _GogStyle {
+	GOStyle base;
+};
 typedef GObjectClass GogStyleClass;
 GType gog_style_get_type (void);
 GSF_CLASS (GogStyle, gog_style,
